package org.mai806.catalog.dao.jdbc;

import org.mai806.catalog.model.Author;
import org.mai806.catalog.model.CatalogItem;
import org.mai806.catalog.dao.Session;

import java.sql.ResultSet;
import java.sql.PreparedStatement;

/**
 *      Author
 */
public class AuthorDAO extends BaseDAO {

    public AuthorDAO(Session session) {
        super(session);
    }

    public Author getAuthorByItem(CatalogItem item) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            stmt = getConnection().prepareStatement("select ID, NAME, BIRTHDATE from AUTHOR where ID in "
                    +"(select AUTHOR_ID from CATALOG_ITEM I where I.ID=?)");
            stmt.setLong(1, item.getId());
            rs = stmt.executeQuery();
            if (rs.next()) {
                return readAuthor(rs);
            }
        } finally {
            close(stmt, rs);
        }
        return null;
    }

     public Author saveAuthor(Author author) throws Exception {
        PreparedStatement stmt = null;
        try {
            if (author.getId()==null) { // 
                long newID = generateID();
                author.setId(newID);
                stmt = getConnection().prepareStatement("insert into AUTHOR(NAME,BIRTHDATE,ID) values(?,?,?)");
            } else { //  
                stmt = getConnection().prepareStatement("update CATALOG set NAME=?, BIRTHDATE=? where ID=?");
            }
            stmt.setString(1, author.getName());
            stmt.setDate(2, new java.sql.Date(author.getBirthdate().getTime()) );
            stmt.setLong(3, author.getId());
            stmt.execute();
        } finally {
            close(stmt);
        }
        return author;
    }

    private Author readAuthor(ResultSet rs)
        throws Exception {
        Author author = new Author();
        author.setId(rs.getLong("ID"));
        author.setName(rs.getString("NAME"));
        author.setBirthdate(rs.getDate("BIRTHDATE"));
        return author;
    }
}
