package org.mai806.catalog.dao.jdbc;

import java.util.List;
import java.util.ArrayList;
import java.sql.ResultSet;
import java.sql.SQLException;
import java.sql.PreparedStatement;

import org.mai806.catalog.model.Catalog;
import org.mai806.catalog.model.CatalogItem;
import org.mai806.catalog.dao.Session;

/**
 *      Catalog
 */
public class CatalogDAO extends BaseDAO {

    public CatalogDAO(Session session) {
        super(session);
    }

    /**
     *     
     * @param id  
     * @return 
     */
    public Catalog getCatalog(Long id) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            stmt = getConnection().prepareStatement("select ID, NAME from CATALOG where ID=?");
            stmt.setLong(1, id);
            rs = stmt.executeQuery();
            if (rs.next()) {
                return readCatalog(rs);
            }
        } finally {
            close(stmt, rs);
        }
        return null;
    }

    public Catalog getCatalogByItem(CatalogItem item) throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            stmt = getConnection().prepareStatement("select ID, NAME from CATALOG where ID in " +
                    "(select CATALOG_ID from CATALOG_ITEM I where I.ID=?)");
            stmt.setLong(1, item.getId());
            rs = stmt.executeQuery();
            if (rs.next()) {
                return readCatalog(rs);
            }
        } finally {
            close(stmt, rs);
        }
        return null;
    }

    /**
     *    
     * @return  
     */
    public List<Catalog> getCatalogs() throws Exception {
        PreparedStatement stmt = null;
        ResultSet rs = null;
        List<Catalog> list = new ArrayList<Catalog>();
        try {
            stmt = getConnection().prepareStatement("select ID, NAME from CATALOG");
            rs = stmt.executeQuery();
            while (rs.next()) {
                list.add( readCatalog(rs) );
            }
        } finally {
            close(stmt, rs);
        }
        return list;
    }

    /**
     *  (  )
     *       
     * @param catalog
     * @return  
     */
    public Catalog saveCatalog(Catalog catalog) throws Exception {
        PreparedStatement stmt = null;
        try {
            if (catalog.getId()==null) { // 
                long newID = generateID();
                catalog.setId(newID);
                stmt = getConnection().prepareStatement("insert into CATALOG(NAME,ID) values(?,?)");
            } else { //  
                stmt = getConnection().prepareStatement("update CATALOG set NAME=? where ID=?");
            }
            stmt.setString(1, catalog.getName());
            stmt.setLong(2, catalog.getId());
            stmt.execute();
        } finally {
            close(stmt);
        }
        return catalog;
    }

    /**
     *  
     */
    public void deleteCatalog(Catalog catalog) throws Exception {
        PreparedStatement stmt = null;
        try {
            stmt = getConnection().prepareStatement("delete from CATALOG where ID=?");
            stmt.setLong(1, catalog.getId());
            stmt.execute();
        } finally {
            close(stmt);
        }
    }

    private Catalog readCatalog(ResultSet rs) throws SQLException {
        Catalog catalog = new Catalog();
        catalog.setId( rs.getLong("ID") );
        catalog.setName( rs.getString("NAME") );
        return catalog;
    }

}
