package org.mai806.sample.mvc;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;
import java.util.*;

/**
 * ,     Web-   *.html
 * : FrontController
 */
public class DispatcherServlet extends HttpServlet {

    private Map<String,Controller> controllers = new HashMap<String,Controller>();
    private Map<String,String> views = new HashMap<String,String>();

    /**
     *  
     * @param servletConfig
     * @throws ServletException
     */
    public void init(ServletConfig servletConfig) throws ServletException {
        try {
            //    
            ResourceBundle controllersProps = ResourceBundle.getBundle("controllers");
            for (Enumeration<String> en = controllersProps.getKeys(); en.hasMoreElements();) {
                String key = en.nextElement();
                String controllerClass = controllersProps.getString(key);
                //   
                Controller controllerObj = (Controller)Class.forName(controllerClass).newInstance();
                controllers.put(key, controllerObj);
            }

            //    
            ResourceBundle viewsProps = ResourceBundle.getBundle("views");
            for (Enumeration<String> en = viewsProps.getKeys(); en.hasMoreElements();) {
                String key = en.nextElement();
                views.put(key, viewsProps.getString(key));
            }
        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException(e);
        }
    }

    protected void doGet(HttpServletRequest httpServletRequest,
                         HttpServletResponse httpServletResponse)
            throws ServletException, IOException {
        process(httpServletRequest, httpServletResponse);
    }

    protected void doPost(HttpServletRequest httpServletRequest,
                          HttpServletResponse httpServletResponse)
            throws ServletException, IOException {
        process(httpServletRequest, httpServletResponse);
    }

    /**
     *  
     * @param req
     * @param resp
     * @throws ServletException
     * @throws IOException
     */
    protected void process(HttpServletRequest req, HttpServletResponse resp)
            throws ServletException, IOException {

        String url = req.getServletPath();
        System.out.println("Looking for controller by URL: "+url);
        //   .   URL
        Controller controller = controllers.get( url );
        if (controller==null) {
            throw new ServletException("Controller for "+url+" not found");
        }
        System.out.println("Found controller "+controller.getClass());

        try {
            //   
            ModelAndView modelAndView = controller.processRequest(req, resp);

            //     ,       JSP
            for (Map.Entry<String,Object> modelItem : modelAndView.getModel().entrySet()) {
                req.setAttribute(modelItem.getKey(), modelItem.getValue());
            }

            //  
            String view = views.get( modelAndView.getViewId() );
            if (view==null || view.trim().length()==0) {
                throw new Exception("Controller doesn't return view");
            }
            //     (forward)
            req.getRequestDispatcher(view).forward(req, resp);

        } catch (Exception e) {
            e.printStackTrace();
            throw new ServletException(e);
        }

    }
}