package org.mai806.sample.servlet;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import javax.servlet.ServletConfig;
import java.io.IOException;
import java.io.OutputStream;
import java.io.InputStream;
import java.sql.*;

/**
 * Fetches images from DB by given path from URL
 */
public class ImageServlet extends HttpServlet {
    private String base_uri = "/";

    public void init(ServletConfig servletConfig) throws ServletException {
        base_uri = servletConfig.getInitParameter("base_uri");
    }

    protected void doGet(HttpServletRequest req,
                         HttpServletResponse resp)
            throws ServletException, IOException {

        // Calculating file path to retrieve file from database,
        // cut application context prefix and servlet base prefix
        String uri = req.getRequestURI();
        String prefix = req.getContextPath() + base_uri;
        String filePath = uri.substring(prefix.length() + 1);

        try {
            // Retrieving file and writing it to response
            writeFileToResponse(filePath, resp);
        } catch (Exception e) {
            throw new ServletException(e);
        }
    }

    /**
     * Retrieves file from database table BLOB field
     * and writes it to servlet response stream
     * @param filePath
     * @param resp
     * @throws Exception
     */
    private void writeFileToResponse(String filePath, HttpServletResponse resp)
            throws Exception {

        Connection conn = null;
        PreparedStatement stmt = null;
        ResultSet rs = null;
        try {
            conn = Connector.getConnection();
            stmt = conn.prepareStatement("select contenttype, content from FILES where name=?");
            stmt.setString(1, filePath);
            rs = stmt.executeQuery();
            if (rs.next()) {
                // setting content-type
                resp.setContentType(rs.getString(1));
                // Copying blob stream to servlet stream
                copyStream(resp.getOutputStream(), rs.getBinaryStream(2));
            } else {
                throw new Exception("File " + filePath + " not found");
            }
        } finally {
            if (rs != null) { rs.close(); }
            if (stmt != null) { stmt.close(); }
            if (conn != null) { conn.close(); }
        }
    }

    /**
     * Copies content from input stream to output stream and closes both streams
     * @param out output stream
     * @param in input stream
     * @throws IOException
     */
    private void copyStream(OutputStream out, InputStream in) throws IOException {
        int value;
        while ((value = in.read()) != -1) {
            out.write(value);
        }
        out.close();
        in.close();
    }
}
