package org.mai806.sample.servlet;

import org.apache.commons.fileupload.servlet.ServletFileUpload;
import org.apache.commons.fileupload.FileItemFactory;
import org.apache.commons.fileupload.FileItem;
import org.apache.commons.fileupload.disk.DiskFileItemFactory;

import javax.servlet.http.HttpServlet;
import javax.servlet.http.HttpServletRequest;
import javax.servlet.http.HttpServletResponse;
import javax.servlet.ServletException;
import java.io.IOException;
import java.io.InputStream;
import java.util.List;
import java.sql.Connection;
import java.sql.PreparedStatement;

/**
 * Uploads file to database.
 */
public class UploadServlet extends HttpServlet {
    protected void doPost(HttpServletRequest request,
                          HttpServletResponse response)
            throws ServletException, IOException {
        try {
            FileItemFactory factory = new DiskFileItemFactory();

            // Create a new file upload handler
            ServletFileUpload upload = new ServletFileUpload(factory);

            // Parse the request
            List<FileItem> items = upload.parseRequest(request);

            // Gathering parameters
            String filePath = null;
            String contentType = null;
            InputStream stream = null;
            long length = 0;
            for (FileItem item : items) {
                if (item.isFormField() && item.getFieldName().equals("filePath")) {
                    filePath = item.getString();
                } else if (!item.isFormField()) {
                    contentType = item.getContentType();
                    stream = item.getInputStream();
                    length = item.getSize();
                }
            }
            if (filePath==null || stream==null) {
                throw new Exception("No file uploaded");
            }

            // Saving file to DB
            saveFile(filePath, contentType, stream, length);

        } catch (Exception e) {
            throw new ServletException(e);
        }
    }

    private void saveFile(String filePath, String contentType, InputStream stream, long length)
        throws Exception {
        System.out.println("Saving file "+filePath+" of "+contentType+" to DB");
        Connection conn = null;
        PreparedStatement stmt = null;
        try {
            conn = Connector.getConnection();
            stmt = conn.prepareStatement("insert into FILES(id, name, contenttype, content) "
                    + "values (SEQ_ID.nextval,?,?,?)");
            stmt.setString(1, filePath);
            stmt.setString(2, contentType);
            stmt.setBinaryStream(3, stream, (int) length);
            int rowsAffected = stmt.executeUpdate();
            System.out.println("RowsAffected=" + rowsAffected);
        } finally {
            if (conn!=null) {
                conn.close();
            }
            if (stmt != null) {
                stmt.close();
            }
        }
    }
}
